from datetime import datetime, timedelta, timezone
from typing import TypedDict, Any, List, Dict
from db_utils import get_player, insert_player, delete_player, insert_pull, get_last_rolled_at, \
        get_random_character
from add_character import add_character
from config import GACHA_ROLL_INTERVAL
from custom_types import BotResponse, ParsedNotification


def do_roll(author: str) -> BotResponse:
    '''Determines whether the user can roll, then pulls a random character'''
    user_id = get_player(author)
    if not user_id:
        return {
        'message':f'{author} 🛑 You haven’t signed up yet! Use the `signup` command to start playing.',
        'attachment_urls': None
        }
    # Get date of user's last roll
    date = get_last_rolled_at(user_id)

    # No date means it's users first roll
    if date:
        # SQLite timestamps returned by the DB are always in UTC
        # Below timestamps are to be converted to UTC
        prev = datetime.strptime(str(date) + '+0000', '%Y-%m-%d %H:%M:%S%z')
        now = datetime.now(timezone.utc)

        time_since_last_roll = now - prev
        roll_interval = timedelta(seconds=GACHA_ROLL_INTERVAL)
        duration = roll_interval - time_since_last_roll

        # User needs to wait before they can roll again
        if time_since_last_roll < roll_interval:
            remaining_duration = None
            if duration.seconds > 3600:
                remaining_duration = f'{-(duration.seconds // -3600)} hours'
            elif duration.seconds > 60:
                remaining_duration = f'{-(duration.seconds // -60)} minutes'
            else:
                remaining_duration = f'{duration.seconds} seconds'

            return {
                'message': f'{author} ⏱️ Please wait another \
{remaining_duration} before rolling again.',
                'attachment_urls': None
            }

    character = get_random_character()

    if not character:
        return {
            'message': f'{author} Uwaaa... something went wrong! No \
characters found. 😿',
            'attachment_urls': None
        }

    insert_pull(user_id, character['id'])
    stars = '⭐️' * character['rarity']
    return {
        'message': f'{author} 🎲 Congrats! You rolled {stars} \
**{character['name']}**\nShe\'s all yours now~ 💖✨',
        'attachment_urls': [character['image_url']]
    }

def do_signup(author: str) -> BotResponse:
    '''Registers a new user if they haven’t signed up yet.'''
    user_id = get_player(author)

    if user_id:
        return {
            'message':f'{author} 👀 You’re already signed up! Let the rolling begin~ 🎲',
            'attachment_urls': None
        }

    new_user_id = insert_player(author)
    return {
            'message': f'{author} ✅ Signed up successfully! Your gacha destiny begins now... ✨ Use the roll command to start!',
            'attachment_urls': None
        }

def is_float(val: Any) -> bool:
    '''Returns true if `val` can be converted to a float'''
    try:
        float(val)
        return True
    except ValueError:
        return False


def do_create(
        author: str,
        arguments: List[str],
        note_obj: Dict[str, Any]) -> BotResponse:
    '''Creates a character'''
    # Example call from bot logic
    image_url = note_obj.get('files', [{}])[0].get('url') \
        if note_obj.get('files') else None

    if not image_url:
        return {
            'message': f'{author} You need an image to create a character, \
dumbass.',
            'attachment_urls': None
        }

    if len(arguments) != 2:
        return {
            'message': f'{author} Please specify the following attributes \
in order: name, rarity',
            'attachment_urls': None
        }

    if not (arguments[1].isnumeric() and 1 <= int(arguments[1]) <= 5):
        return {
            'message': f'{author} Invalid rarity: \'{arguments[1]}\' must \
be a number between 1 and 5',
            'attachment_urls': None
        }
    if not (is_float(arguments[2]) and 0.0 < float(arguments[2]) <= 1.0):
        return {
            'message': f'{author} Invalid drop weight: \'{arguments[2]}\' \
must be a decimal value between 0.0 and 1.0',
            'attachment_urls': None
        }

    character_id, file_id = add_character(
        name=arguments[0],
        rarity=int(arguments[1]),
        image_url=image_url
    )
    return {
        'message': f'{author} Added {arguments[0]}, ID {character_id}.',
        'attachment_urls': [file_id]
    }


def do_help(author: str) -> BotResponse:
    '''Provides a list of commands that the bot can do.'''
    return {
        'message':f'{author} Here\'s what I can do:\n \
            - `roll` Pulls a random character.\
            - `create <name> <rarity>` Creates a character using a given image.\
            - `signup` Registers your account.\
            - `delete_account` Deletes your account.\
            - `help` Shows this message',
            'attachment_urls': None
    }
            
def delete_account(author: str) -> BotResponse:
    return {
        'message':f'{author} ⚠️ This will permanently delete your account and all your cards.\n'
        'If you’re sure, reply with `confirm_delete` to proceed.\n\n'
        '**There is no undo.** Your gacha luck will be lost to the void... 💀✨',
        'attachment_urls': None

    }

def confirm_delete(author: str) -> BotResponse:
    
    delete_player(author)

    return {
        'message':f'{author} 🧼 Your account and all your cards have been deleted. RIP your gacha history 🕊️✨',
        'attachment_urls': None
    }


def generate_response(notification: ParsedNotification) -> BotResponse | None:
    '''Given a command with arguments, processes the game state and
    returns a response'''

    # Temporary response variable
    res: BotResponse | None = None
    # TODO: Check if the user has an account
    author = notification['author']
    user_id = get_player(author)
    command = notification['command']
    # Check if the user is an administrator
    # user_is_administrator = user_is_administrator()

    # Unrestricted commands
    match command:
        case 'signup':
            res = do_signup(author)
        case 'help':
            res = do_help(author)
        case 'roll':
            res = do_roll(author)
        case _:
            pass

    if not user_id:
        return res

    # User commands
    match command:
        case 'create':
            res = do_create(
                author,
                notification['arguments'],
                notification['note_obj']
            )
        case 'signup':
            res = do_signup(author)
        case 'delete_account':
            res = delete_account(author)
        case 'confirm_delete':
            res = confirm_delete(author)
        case _:
            pass
    # if not user_is_administrator:
    return res

    # Administrator commands go here
